#include <stdio.h>
#include "sofa.h"
#include "sofam.h"

/* Arcseconds to radians */
#define DAS2R (4.848136811095359935899141e-6)

/*  Function prototypes. */
void C2iexample2 ( double tt1, double tt2, double dx, double dy, 
                      double *x, double *y, double *s, double c[3][3] );     
void C2iexample3 ( double tt1, double tt2, double dx, double dy, 
                   double *x, double *y, double *s, double c[3][3] );   
void Pmat ( char text[], double r[3][3] );
void Pxys ( char text[], double x, double y, double s );
void PDxys ( char text[], double dx, double dy, double ds );
void repjd ( char s[], double t1, double t2 );


int main()
/*
   PROGRAM SOFA EARTH ORIENTATION EXAMPLES
  
   Demonstration program of SOFA Routines for Earth Orientation
   made up of 4 examples routines for the Journees 2013 Workshop
  
   NB Currently all IERS parameters are set to zero.

  This program does not follow any strict coding standards and
  is not part of the SOFA code.
  
   Catherine Hohenkerk
   Chair, IAU SOFA Board
   2013 September 8
   
   Completed and debugged by John Bangert, 2013 Sep 10.
*/
{
   int iy, im, id, ihr, imn;
   double  sec, dx, dy;
   double tt1, tt2, x1, y1, s1, c1[3][3], x2, y2, s2, c2[3][3];

   printf ( "+ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -+\n" );
   printf ( "2013 Journees Workshop, Earth Orientation using IAU SOFA Routines\n" );
   printf ( "+ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -+\n\n" );
   
/*
   User parameters for Examples 2 and 3

   Civil date, TT
*/
   iy = 2013;
   im = 9;   
   id = 15;
   ihr = 17;
   imn = 30;
   sec = 0e0;

/* IERS CIP offsets wrt IAU 2006/2000A (radians)  */
   dx =  -0.20e-6 * DAS2R;     
   dy =  -0.10e-6 * DAS2R;    

   printf ( "+ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -+\n" );
   printf ("GCRS -> CIRS, 3 Examples using SOFA ANSI C Routines\n");
   printf ( "+ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -+\n\n" );

   printf ( "Date and time (TT) %4.4d %2.2d %2.2d %2.2d:%2.2d:%05.2f\n", iy, im, id, ihr, imn, sec);  

   if ( iauDtf2d ( "TT", iy, im, id, ihr, imn, sec, &tt1, &tt2 ) ) return -1;

   printf ("\nIERS Parameters: \n");
   printf ( "      DX (\") = %.9f   DY (\") = %.9f\n",  dx/DAS2R, dy/DAS2R );
   printf ( "+ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -+\n" );
   
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

   C2iexample2 ( tt1, tt2, dx, dy, &x1, &y1, &s1, c1 );
   printf ( "+ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -+\n" );

   C2iexample3 ( tt1, tt2, dx, dy, &x2, &y2, &s2, c2 );
   printf ( "+ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -+\n" );
     
/* Differences */
   PDxys ( "Differences using iau_XY06 and iau_XYS06A", x1-x2, y1-y2, s1-s2 );
   printf ( "+ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -+\n");
   
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
   printf ( "+ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -+\n");
   printf ( "Produced by Catherine Hohenkerk, Chair, IAU SOFA Board, 2013 September\n");   
   printf ( "+ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -+\n");
   
   return (0);
}
      
void C2iexample2 ( double tt1, double tt2, double dx, double dy, 
                      double *x, double *y, double *s, double c[3][3] )     
/*
   Example 2: SOFA Demonstration routine for the Journees 2013 Workshop
  
    Form the celestial-to-intermediate matrix for a given date explicitly
    using the series representation for X and Y (IAU 2006 precession, 
    IAU 2000A nutation), and the CIO locator s.
  
    Given:
       tt1,tt2    d       TT as a 2-part Julian Date
       dx, dy     d       IERS CIP offsets wrt IAU 2006/2000A
  
    Returned:
       x,y        d       Celestial Intermediate Pole
       s          d       the CIO locator s
       c         d(3,3)  celestial-to-intermediate matrix
  
    Notes:
  
    1) The TT date tt1+tt2 is a Julian Date, apportioned in any
       convenient way between the two arguments (see SOFA routines).
  
    2) dx, dy, x, y, s are all in radians.
*/
{
   printf ("\nExample 2\n");
   repjd ( " JD (TT) = TT1 + TT2 = ", tt1, tt2 );  
   printf ("\n");

/*
   Step 1: - Given the TT instant calculate the X, Y of the CIP using the
   series formulation (IAU 2006/2000A) for CIP, and the CIO locator s.
*/
   iauXy06 ( tt1, tt2, x, y );                                      
   *s = iauS06 ( tt1, tt2, *x, *y );
   
   Pxys ( "X, Y, s using iauXy06 and iauS06", *x, *y, *s );
   Pxys ( "X+DX, Y+DY, s", *x+dx, *y+dy, *s );

/*
   Step 2: Incorporating the IERS corrections dX, dY calculate the 
   matrix c, the GCRS to CIRS matrix.
*/
   iauC2ixys ( *x+dx, *y+dy, *s, c );
   Pmat ( "GCRS TO CIRS using iauC2ixys", c );

   return;
}
      
void C2iexample3 ( double tt1, double tt2, double dx, double dy, 
                   double *x, double *y, double *s, double c[3][3] )   
/*
   Example 3:  SOFA Demonstration routine for the Journees 2013 Workshop
  
    Form the celestial-to-intermediate matrix for a given date using 
    bias-precession-nutation angles (IAU 2006 precession and IAU 2000A
    nutation) for X and Y, and the CIO locator s.
  
    Given:
       tt1,tt2    d       TT as a 2-part Julian Date
  
    Returned:
       dx, dy     d       IERS CIP offsets wrt IAU 2006/2000A 
       x,y        d       Celestial Intermediate Pole
       s          d       the CIO locator s
       c          d(3,3)  celestial-to-intermediate matrix
  
    Notes:
  
    1) The TT date tt1+tt2 is a Julian Date, apportioned in any
       convenient way between the two arguments (see SOFA routines).
  
    2) dx, dy, x, y, s are all in radians.
*/
{
   printf ("\nExample 3\n");
   repjd ( " JD (TT) = TT1 + TT2 = ", tt1, tt2 );  
   printf ("\n");

/*
   Step 1: SOFA generated X, Y via IAU 2006/2000A bias-precession-nutation 
   angles
*/
   iauXys06a ( tt1, tt2, x, y, s );
         
   Pxys ( "X, Y, s using iauXys06a", *x, *y, *s );
   Pxys ( "X+DX, Y+DY, s", *x+dx, *y+dy, *s );
      
/*
   Step 2: Incorporating the IERS corrections dX, dY calculate the 
   matrix c, the GCRS to CIRS matrix.
*/
   iauC2ixys ( *x+dx, *y+dy, *s, c ); 
   Pmat ( "GCRS TO CIRS matrix using iauC2ixys", c );
   
   return;
}
      
void Pmat ( char text[], double r[3][3] )
/* Print matrix R */
{
   printf ( "%s\n", text );
   printf ( "     %19.15f %19.15f %19.15f\n", r[0][0], r[0][1], r[0][2] );
   printf ( "     %19.15f %19.15f %19.15f\n", r[1][0], r[1][1], r[1][2] );
   printf ( "     %19.15f %19.15f %19.15f\n", r[2][0], r[2][1], r[2][2] );
    
   return;
}

void Pxys ( char text[], double x, double y, double s )
/* Print X, Y, s in arc seconds  */
{      
   printf ( "%s\n", text );
   printf ( "   X (\") = %13.7f\n", x/DAS2R );
   printf ( "   Y (\") = %13.7f\n", y/DAS2R );
   printf ( "   s (\") = %13.7f\n", s/DAS2R );
   printf ("\n");
   
   return;
}

void PDxys ( char text[], double dx, double dy, double ds )
/* Print differences in X, Y, s in arc seconds  */
{      
   printf ( "%s\n", text );
   printf ( "    X1 - X2 (\") = %11.8f\n", dx/DAS2R );
   printf ( "    Y1 - Y2 (\") = %11.8f\n", dy/DAS2R );
   printf ( "    s1 - s2 (\") = %14.11f\n", ds/DAS2R );

   printf ("\n");
   
   return;
}

void repjd ( char s[], double t1, double t2 )
{
/* Print Julian date  */
   printf ( " %8s", s);
   printf ("%.1f + %.10f = %.10f\n", t1, t2, t1+t2);
   
   return;
}
