      PROGRAM SOFA EARTH ORIENTATION EXAMPLES
*
* Demonstration program of SOFA Routines for Earth Orientation
*
* Example 4 Transformation from the GCRS to the TIRS & ITRS (C2T)
*         1) A 1-CALL process,
*         2) Several CALL process (without and with polar motion).
*
* This program does not follow any strict coding standards and
* is not part of the SOFA code.
*
* Catherine Hohenkerk
* Chair, IAU SOFA Board
* 2013 September 15
*
      IMPLICIT NONE

      DOUBLE PRECISION UTC1, UTC2, TAI1, TAI2, TT1, TT2, UT1A, UT1B
      DOUBLE PRECISION SEC, DX, DY, XP, YP, DUT

      DOUBLE PRECISION X, Y, S, C(3,3), ERA, EO, SP, W(3,3)
      DOUBLE PRECISION Q0(3,3), Q1(3,3)

* SOFA functions
      DOUBLE PRECISION iau_S06, iau_ERA00, iau_EO06A, iau_SP00

      INTEGER          IY, IM, ID, IHR, IMN, J

* Arc seconds to radians
      DOUBLE PRECISION DAS2R
      PARAMETER      ( DAS2R = 4.848136811095359935899141D-6 )
* Degrees to radians
      DOUBLE PRECISION DD2R
      PARAMETER      ( DD2R = 1.745329251994329576923691D-2 )

* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

* Civil date, UTC
      IY = 2013
      IM = 9
      ID = 15
      IHR = 17
      IMN = 30
      SEC = 0D0

* IERS CIP offsets wrt IAU 2006/2000A (guess?) ("->radians)
      DX = -0.2D-06 * DAS2R 
      DY = -0.1D-06 * DAS2R 

* IERS predicted polar motion, coordinates of CIP wrt ITRS ("->radians)
      XP =  +0.1574D0 * DAS2R 
      YP =  +0.3076D0 * DAS2R 

* UT1-UTC (seconds) IERS predicted
      DUT = +0.02792D0

      CALL iau_DTF2D ( 'UTC', IY, IM, ID, IHR, IMN, SEC,
     :                                                 UTC1, UTC2, J )
* UTC -> TAI -> TT
      CALL iau_UTCTAI ( UTC1, UTC2, TAI1, TAI2, J )
      CALL iau_TAITT  ( TAI1, TAI2, TT1,  TT2,  J )

* UTC -> UT1 using IERS DUT = UT1-UTC
      CALL iau_UTCUT1 ( UTC1, UTC2, DUT, UT1A, UT1B, J )

* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
* Method A - Transformation matrix Q0 from GCRS to ITRS (C2T).
* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

* A 1-CALL example using SOFA (IAU 2006/2000A) generated
* quantities given the TT and UT1 instant.

      CALL iau_C2T06A ( TT1, TT2, UT1A, UT1B, XP, YP, Q0 )

* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
* Method B - Transformation matrix from GCRS to TIRS & ITRS C2T).
* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

* GCRS to CIRS matrix (IAU 2006/2000A) using series formulation for
* X, Y of CIP (Step 1 & 2 are equivalent to Example 2, alternatively
* they could be replaced by Example 3)

* Step 1 - generate X, Y and s

      CALL iau_XY06 ( TT1, TT2, X, Y )
      S = iau_S06 ( TT1, TT2, X, Y )

* Step 2: Incorporating the IERS corrections DX, DY calculate
* the matrix C, the GCRS to CIRS matrix.

      CALL iau_C2IXYS ( X+DX, Y+DY, S, C )

* Step 3: - Earth rotation agnle (function of UT1) that rotates from
* CIRS to TIRS & equation of the origins (EO)

      ERA = iau_ERA00 ( UT1A, UT1B )
      
      EO = iau_EO06A ( TT1, TT2 )

* Step 4 - s' TIO Locator & polar motion (wobble), rotation from TIRS to ITRS

      SP = iau_SP00 ( TT1, TT2 )
      CALL iau_POM00 ( XP, YP, SP, W )

* Step 5: - Transformation from CIRS (matrix C) to ITRS

      CALL iau_C2TCIO ( C, ERA, W, Q1 )

* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

* Print results:
      WRITE (6, 935)
      WRITE (6, '(A)')'2013 Journees Workshop, Earth Orientation using I
     :AU SOFA Routines'
      WRITE (6, 935)
      WRITE (6, *)
* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

      WRITE (6, 935)
      WRITE (6, '(A)') 'Example 4: GCRS->TIRS & ITRS, using SOFA Routine
     :s'
      WRITE (6, 935)

* Output various instants
      WRITE (6,950) 'Date and time (UTC)', IY, IM, ID, IHR, IMN, SEC
      WRITE(6,930) 'JD (UTC)', UTC1, UTC2, UTC1+UTC2
      WRITE(6,930) 'JD (TAI)', TAI1, TAI2, TAI1+TAI2
      WRITE(6,930) 'JD (TT) ', TT1,  TT2,  TT1+TT2
      WRITE(6,930) 'JD (UT1)', UT1A, UT1B, UT1A+UT1B

      WRITE(6,'(/A)') 'IERS Parameters:'
      WRITE(6,945) DX/DAS2R, DY/DAS2R, XP/DAS2R, YP/DAS2R, DUT

* METHOD A
      CALL PMAT ( 'GCRS TO ITRS matrix using iau_C2T06A', Q0 )
      WRITE (6, 935)
* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

* METHOD B
      CALL PXYS ( 'X, Y, s using iau_XY06 and iau_S06', X, Y, S )
      CALL PXYS ( 'X+DX, Y+DY, s',                      X+DX, Y+DY, S )
      CALL PMAT ( 'GCRS TO CIRS matrix using iau_C2IXYS', C )
      
      WRITE ( 6, '(2(/A, F17.12, A))') 
     :'Earth rotation angle = ', ERA/DD2R, ' degrees', 
     :'(Equation of origins = ', (EO*1440D0)/(DD2R*15D0),' mins time)'
      WRITE (6, 935)
     
      CALL PMAT ( 'TIRS TO ITRS matrix using iau_SP00 & iau_POM00 (polar
     : motion)',                                          W )
      CALL PMAT ( 'GCRS TO ITRS matrix using iau_C2TCIO', Q1 )
      WRITE (6, 935)
      WRITE (6,935)

*+----------------------------------------------------------------------

 930  FORMAT( 2X, A, ' =',F10.1,' + ' F12.10, ' = ', F18.10)
 935  FORMAT('+ ',33('- '), '-+')
 945  FORMAT( '       DX (") = ', F13.9, 3X,'DY (") = ' F13.9,/
     :        '       XP (") = ', F10.6, 6X,'YP (") = ' F10.6,/
     :        2X, 'UT1-UTC (s) = ', F10.6)
 950  FORMAT(/A, 1X, I4,3(1X,I2.2),':',I2.2,':',F5.2)

      
      END

      SUBROUTINE PMAT ( TEXT, R )

* Print the matrix R

      IMPLICIT NONE
      DOUBLE PRECISION R(3,3)
      INTEGER          I, J
      CHARACTER        TEXT*(*)

      WRITE (6,'(/A)') TEXT
      DO I = 1,3
       WRITE (6, 950) (R(I,J),J=1,3)
 950   FORMAT( 4X, 3F20.15)
      ENDDO

      END

      SUBROUTINE PXYS ( TEXT, X, Y, S )

* Print X, Y, s in arc seconds

      IMPLICIT NONE
      DOUBLE PRECISION X, Y, S
      CHARACTER        TEXT*(*)

*  Arc seconds to radians
      DOUBLE PRECISION DAS2R
      PARAMETER      ( DAS2R = 4.848136811095359935899141D-6 )

      WRITE (6, '(/A)') TEXT
      WRITE (6,950) 'X (") = ', X/DAS2R,
     :              'Y (") = ', Y/DAS2R,
     :              's (") = ', S/DAS2R
950   FORMAT(3X, A, F13.7)

      END

